/*
    terminatorX - realtime audio scratching software
    Copyright (C) 1999, 2000 Alexander Knig
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    File: tX_seqpar.h
 
    Description: Header to tX_seqpar.cc
*/    

#include <pthread.h>
#include <list>

#ifndef _tx_seqpar_h
#define _tx_seqpar_h 1

class tX_seqpar
{	
	protected:
	static list <tX_seqpar *> all;
	static list <tX_seqpar *> update;
	static pthread_mutex_t update_lock;
		
	int touched;
	int gui_active;
	void *vtt; /* have to make this void as tX_vtt.h includes this */
	
	float fwd_value;
	
	u_int32_t persistence_id;
	u_int32_t touch_timestamp;		
	void *last_event_recorded;

	public:
	tX_seqpar(void *);
	tX_seqpar();
	~tX_seqpar();
	
	void set_vtt(void *);
	int is_touched() {return touched;}
	int is_untouched() {return !touched;}
	
	void do_touch(); /* can't be inline as requires tX_sequencer.h */
	void touch() { if (!touched) do_touch();}
	void untouch() { touched=0; last_event_recorded=NULL; }
	
	static void untouch_all();
	
	int is_gui_active() { return gui_active; }
	u_int32_t get_touch_timestamp() { return touch_timestamp; }
	
	static void create_persistence_ids();
	u_int32_t get_persistence_id() { return persistence_id; }
	unsigned int set_persistence_id(u_int32_t pid) { persistence_id=pid; }
	static tX_seqpar *get_sp_by_persistence_id(int pid);

	void record_value (const float value);
	virtual void do_exec(const float value);
	virtual void exec_value(const float value);	
	virtual void do_update_graphics();
	void update_graphics();
	static void update_all_graphics();
	
	/* slot for gui actions */
	void receive_gui_value(const float value); 
	
	/* slot for fastforwarding (no effect for vtt) */
	virtual void receive_forward_value(const float value);
	
	/* slot for mouse and keyboard actions (updating graphics) */
	void receive_input_value(const float value); 
	
	/* Make it so ;) */
	static void materialize_forward_values();

        /* info functions for the editor */
        char *get_vtt_name();
        virtual char *get_name(); 
};

class tX_seqpar_update : public tX_seqpar
{
	virtual void exec_value(const float value);	
};

class tX_seqpar_no_update : public tX_seqpar
{
	virtual void exec_value(const float value);	
};

class tX_seqpar_update_active_forward : public tX_seqpar_update
{
	virtual void receive_forward_value(const float value);
};

class tX_seqpar_no_update_active_forward : public tX_seqpar_no_update
{
	virtual void receive_forward_value(const float value);
};

/* now the classes for *real* sequenceable parameters */

class tX_seqpar_master_volume : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_master_pitch : public tX_seqpar_update_active_forward
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_speed : public tX_seqpar_no_update_active_forward
{
	virtual void do_exec(const float value);
        virtual char *get_name(); 
};

class tX_seqpar_spin : public tX_seqpar_no_update_active_forward
{
	virtual void do_exec(const float value);
        virtual char *get_name(); 
};

class tX_seqpar_vtt_volume : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_pitch : public tX_seqpar_update_active_forward
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_trigger : public tX_seqpar_no_update_active_forward
{
	virtual void do_exec(const float value);
        virtual char *get_name(); 
};

class tX_seqpar_vtt_loop : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_sync_client : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_sync_cycles : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_lp_enable : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_lp_gain : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_lp_reso : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_lp_freq : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_ec_enable : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_ec_length : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_ec_feedback : public tX_seqpar_update
{
	virtual void do_exec(const float value);
	virtual void do_update_graphics();	
        virtual char *get_name(); 
};

class tX_seqpar_vtt_mute : public tX_seqpar_no_update
{
	virtual void do_exec(const float value);
        virtual char *get_name(); 
};
#endif 




